from abc import ABC, abstractmethod

import numpy as np

from . import Keyboard, GridLayer, CharactersLayer
from .Layer import Layer


class GUIFacade(ABC):

    @property
    @abstractmethod
    def screenWidth(self) -> int:
        raise NotImplementedError()

    @property
    @abstractmethod
    def screenHeight(self) -> int:
        raise NotImplementedError()

    @abstractmethod
    def createWindow(self, title: str, width: int, height: int):
        raise NotImplementedError()

    @property
    @abstractmethod
    def closingRequested(self) -> bool:
        raise NotImplementedError()

    @closingRequested.setter
    @abstractmethod
    def closingRequested(self, b: bool):
        raise NotImplementedError()

    @abstractmethod
    def createGridLayer(self) -> GridLayer:
        raise NotImplementedError()

    @abstractmethod
    def createCharactersLayer(self) -> CharactersLayer:
        raise NotImplementedError()

    @abstractmethod
    def init(self):
        raise NotImplementedError()

    @abstractmethod
    def updateInputs(self):
        raise NotImplementedError()

    @abstractmethod
    def render(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def keyboard(self) -> Keyboard:
        raise NotImplementedError()

    @abstractmethod
    def setTranslation(self, x: float, y: float):
        raise NotImplementedError()

    @abstractmethod
    def captureScreenContent(self, x1: int, y1: int, x2: int, y2: int) -> np.ndarray:
        raise NotImplementedError()

    @abstractmethod
    def quit(self):
        raise NotImplementedError()
