import time
from typing import List

import pygame
from pygame.event import Event

from gui import Keyboard, KeyEvent
from gui.pygame import PygameKeyEvent


class PygameKeyboard(Keyboard):

    def __init__(self):
        self.__keys = [False] * Keyboard.K_MAXCODE  # type: List[bool]
        self.__events = []  # type: List[PygameKeyEvent]

    def isKeyPressed(self, key: int) -> bool:
        if key < 0 or key >= len(self.__keys):
            return False
        return self.__keys[key]

    @property
    def keyEvents(self) -> List[KeyEvent]:
        return self.__events

    def clearKeyEvents(self):
        self.__events.clear()

    def parsePygameEvent(self, event: Event):
        if event.type == pygame.KEYDOWN:
            if event.key not in pygameKeyCode2FacadeCode:
                return
            key = pygameKeyCode2FacadeCode[event.key]
            keyEvent = PygameKeyEvent(KeyEvent.KEYDOWN, key, time.time())
            self.__events.append(keyEvent)
            self.__keys[key] = True
        elif event.type == pygame.KEYUP:
            if event.key not in pygameKeyCode2FacadeCode:
                return
            key = pygameKeyCode2FacadeCode[event.key]
            keyEvent = PygameKeyEvent(KeyEvent.KEYUP, key, time.time())
            self.__events.append(keyEvent)
            self.__keys[key] = False


pygameKeyCode2FacadeCode = {
    pygame.K_LEFT: Keyboard.K_LEFT,
    pygame.K_RIGHT: Keyboard.K_RIGHT,
    pygame.K_UP: Keyboard.K_UP,
    pygame.K_DOWN: Keyboard.K_DOWN,
    pygame.K_ESCAPE: Keyboard.K_ESCAPE
}
