from PIL import Image

from LevelLoader import LevelLoader
from gui import GUIFacadeFactory, KeyEvent, Keyboard

# Load level
levelLoader = LevelLoader("level.tmx")
levelWidth = levelLoader.width
levelHeight = levelLoader.height
groundTileCoords, groundTileset, groundTileWidth, groundTileHeight = levelLoader.decodeLayer(0)
waterTileCoords, waterTileset, waterTileWidth, waterTileHeight = levelLoader.decodeLayer(1)
treeTileCoords, treeTileset, treeTileWidth, treeTileHeight = levelLoader.decodeLayer(2)

# Create window
guiFacade = GUIFacadeFactory().createInstance("OpenGL")
guiFacade.createWindow(
    "OpenGL 2D Facade - https://www.patternsgameprog.com/",
    1280, 768
)

# Ground layer
groundLayer = guiFacade.createGridLayer()
groundLayer.setTileset(groundTileset, groundTileWidth, groundTileHeight)
groundLayer.setTiles(groundTileCoords)

# Water layer
waterLayer = guiFacade.createGridLayer()
waterLayer.setTileset(waterTileset, waterTileWidth, waterTileHeight)
waterLayer.setTiles(waterTileCoords)

# Tree layer
treeLayer = guiFacade.createGridLayer()
treeLayer.setTileset(treeTileset, treeTileWidth, treeTileHeight)
treeLayer.setTiles(treeTileCoords)

# Characters layer
charsLayer = guiFacade.createCharactersLayer()
charsLayer.setTileset("characters.png", 32, 32)
charsLayer.setCharacterCount(1)
charsLayer.setCharacterTile(0, 1, 0)
characterX = 13.0
characterY = 8.0
charsLayer.setCharacterLocation(0, characterX, characterY)
characterSpeed = 1.0 / 2

# Main loop
guiFacade.init()
translationX = 0.0
translationY = 0.0
minTranslationX = 0
minTranslationY = 0
viewWidth = guiFacade.screenWidth / groundTileWidth
viewHeight = guiFacade.screenHeight / groundTileHeight
maxTranslationX = levelWidth - viewWidth
maxTranslationY = levelHeight - viewHeight
while not guiFacade.closingRequested:
    # Update inputs state
    guiFacade.updateInputs()

    # Keyboard key single press
    keyboard = guiFacade.keyboard
    for keyEvent in keyboard.keyEvents:
        if keyEvent.type == KeyEvent.KEYDOWN:
            if keyEvent.key == Keyboard.K_ESCAPE:
                guiFacade.closingRequested = True
                break
    keyboard.clearKeyEvents()

    # Keyboard key multi/continuous press
    if keyboard.isKeyPressed(Keyboard.K_LEFT):
        characterX -= characterSpeed
        charsLayer.setCharacterTile(0, 4, 0)
    if keyboard.isKeyPressed(Keyboard.K_RIGHT):
        characterX += characterSpeed
        charsLayer.setCharacterTile(0, 7, 0)
    if keyboard.isKeyPressed(Keyboard.K_UP):
        characterY -= characterSpeed
        charsLayer.setCharacterTile(0, 10, 0)
    if keyboard.isKeyPressed(Keyboard.K_DOWN):
        characterY += characterSpeed
        charsLayer.setCharacterTile(0, 1, 0)

    if characterX <= 0:
        characterX = 0
    if characterY <= 0:
        characterY = 0
    if characterX > levelWidth - 1:
        characterX = levelWidth - 1
    if characterY > levelHeight - 1:
        characterY = levelHeight - 1

    charsLayer.setCharacterLocation(0, characterX, characterY)

    # View
    if characterX < translationX + 10:
        translationX = characterX - 10
    if characterY < translationY + 10:
        translationY = characterY - 10
    if characterX > translationX + viewWidth - 10:
        translationX = characterX - viewWidth + 10
    if characterY > translationY + viewHeight - 10:
        translationY = characterY - viewHeight + 10

    if translationX < minTranslationX:
        translationX = minTranslationX
    if translationY < minTranslationY:
        translationY = minTranslationY
    if translationX >= maxTranslationX:
        translationX = maxTranslationX
    if translationY >= maxTranslationY:
        translationY = maxTranslationY

    guiFacade.setTranslation(translationX, translationY)

    # Render scene
    guiFacade.render()

# Screen capture (to check that the rendering is ok)
capture = guiFacade.captureScreenContent(0, 0, guiFacade.screenWidth, guiFacade.screenHeight)
Image.fromarray(capture).save("capture.png")

# Delete all objects and quit
guiFacade.quit()
